#
# Rules for all phony targets.
#

.PHONY: all dep depend depclean make \
  check test leaktest \
  clean distclean cvsclean \
  index manhtml indent indentclean \
  doc dist release \
  install uninstall \
  rpm srpm deb

all: $(alltarg)

make:
	echo > $(srcdir)/autoconf/make/filelist.mk~
	echo > $(srcdir)/autoconf/make/modules.mk~
	cd $(srcdir); \
	bash autoconf/scripts/makemake.sh \
	     autoconf/make/filelist.mk~ \
	     autoconf/make/modules.mk~
	sh ./config.status
	
dep depend: $(alldep)
	echo '#' > $(srcdir)/autoconf/make/depend.mk~
	echo '# Dependencies.' >> $(srcdir)/autoconf/make/depend.mk~
	echo '#' >> $(srcdir)/autoconf/make/depend.mk~
	echo >> $(srcdir)/autoconf/make/depend.mk~
	cat $(alldep) >> $(srcdir)/autoconf/make/depend.mk~
	sh ./config.status

clean:
	rm -f $(allobj)
	find . -type f -name "*.c~" -exec rm -f '{}' ';'

depclean:
	rm -f $(alldep)

indentclean:
	cd $(srcdir) && for FILE in $(allsrc); do rm -fv ./$${FILE}~; done

distclean: clean depclean
	rm -f $(alltarg) src/include/config.h
	rm -rf $(package)-$(version).tar* $(package)-$(version) BUILD-DEB
	rm -f *.rpm *.deb
	rm -f *.html config.*
	rm -f test.db trace
	rm -f .test-spam .test-non-spam
	rm -f .testdump-a .testdump-b .testdump-c
	rm -f fakemail gmon.out
	rm Makefile

cvsclean: distclean
	rm -f doc/lsm
	rm -f doc/$(package).spec
	rm -f doc/quickref.1
	rm -f doc/quickref.txt
	rm -f configure
	echo -n > $(srcdir)/autoconf/make/depend.mk~
	echo -n > $(srcdir)/autoconf/make/filelist.mk~
	echo -n > $(srcdir)/autoconf/make/modules.mk~

doc: doc/quickref.txt

index:
	(cd $(srcdir); sh autoconf/scripts/index.sh $(srcdir)) > index.html

manhtml:
	@man2html ./doc/quickref.1 \
	| sed -e '1,/<BODY/d' -e '/<\/BODY/,$$d' \
	      -e 's|<A [^>]*>&nbsp;</A>||ig' \
	      -e 's|<A [^>]*>\([^<]*\)</A>|\1|ig' \
	      -e '/<H1/d' -e 's|\(</H[0-9]>\)|\1<P>|ig' \
	| sed -e '1,/<HR/d' -e '/<H2>Index/,/<HR/d'

indent:
	cd $(srcdir) && indent -npro -kr -i8 -cd42 -c45 $(allsrc)

dist: doc
	rm -rf $(package)-$(version)
	mkdir $(package)-$(version)
	cp -dprf Makefile $(distfiles) $(package)-$(version)
	cd $(package)-$(version); $(MAKE) distclean
	cp -dpf doc/lsm             $(package)-$(version)/doc/
	cp -dpf doc/$(package).spec $(package)-$(version)/doc/
	cp -dpf doc/quickref.txt    $(package)-$(version)/doc/
	chmod 644 `find $(package)-$(version) -type f -print`
	chmod 755 `find $(package)-$(version) -type d -print`
	chmod 755 `find $(package)-$(version)/autoconf/scripts`
	chmod 755 $(package)-$(version)/configure
	chmod 755 $(package)-$(version)/debian/rules
	rm -rf DUMMY `find $(package)-$(version) -type d -name CVS`
	tar cf $(package)-$(version).tar $(package)-$(version)
	rm -rf $(package)-$(version)
	-cat $(package)-$(version).tar \
	 | bzip2 > $(package)-$(version).tar.bz2 \
	 || rm -f $(package)-$(version).tar.bz2
	$(DO_GZIP) $(package)-$(version).tar

.test-spam:
	$(CC) -o fakemail $(srcdir)/autoconf/scripts/fakemail.c
	./fakemail \
	  $(srcdir)/test/tokenlist-non-spam 15 > .test-non-spam
	./fakemail \
	  $(srcdir)/test/tokenlist-spam 15 > .test-spam

check test: $(package) .test-spam
	@rm -f test.db
	@bash -c "echo -n ' 01: Database creation'"
	@formail +1 -1 -s cat < .test-spam \
	  | ./$(package) -d test.db -m -w 2
	@test -e test.db
	@echo '			[OK]'
	@bash -c "echo -n ' 02: Spam mark readback'"
	@ISSPAM=0; formail +1 -1 -s cat < .test-spam \
	  | ./$(package) -d test.db -t || ISSPAM=1; \
	  test $$ISSPAM -eq 1
	@echo '			[OK]'
	@bash -c "echo -n ' 03: Non-spam mark readback'"
	@formail +1 -1 -s cat < .test-non-spam \
	  | ./$(package) -d test.db -M -w 2
	@ISSPAM=0; formail +1 -1 -s cat < .test-non-spam \
	  | ./$(package) -d test.db -t || ISSPAM=1; \
	  test $$ISSPAM -eq 0
	@echo '		[OK]'
	@bash -c "echo -n ' 04: Training mode'"
	@rm -f test.db trace
	@MALLOC_CHECK_=2 \
	  ./$(package) -d test.db -T \
	    .test-spam \
	    .test-non-spam \
	    >/dev/null
	@echo '			[OK]'
	@bash -c "echo -n ' 05: Validity of trained database'"
	@for SPAM in 1 12 3 7 9; do \
	    ISSPAM=0; formail +$$SPAM -1 -s cat < .test-spam \
	    | ./$(package) -d test.db -t || ISSPAM=1; \
	    test $$ISSPAM -eq 1 || exit 1; \
	  done
	@for NONSPAM in 5 13 2 8 14; do \
	    ISSPAM=0; formail +$$NONSPAM -1 -s cat < .test-non-spam \
	    | ./$(package) -d test.db -t || ISSPAM=1; \
	    test $$ISSPAM -eq 0 || exit 1; \
	  done
	@echo '	[OK]'
	@bash -c "echo -n ' 06: Dump and restore'"
	@./$(package) -d test.db -D > .testdump-a
	@./$(package) -d test.db -D | sed 1,2d | sort > .testdump-b
	@rm -f test.db
	@./$(package) -d test.db -R < .testdump-a
	@./$(package) -d test.db -D | sed 1,2d | sort > .testdump-c
	@cmp .testdump-b .testdump-c
	@rm -f .testdump-a .testdump-b .testdump-c
	@echo '			[OK]'
	@bash -c "echo -n ' 07: Automatic database pruning (1)'"
	@./$(package) -d test.db -D > .testdump-a
	@./$(package) -d test.db -D | grep SINCEPRUNE > .testdump-b
	@formail +1 -1 -s cat < .test-non-spam \
	 | ./$(package) -d test.db -m >/dev/null
	@./$(package) -d test.db -D | grep SINCEPRUNE > .testdump-c
	@bash -c "if cmp -s .testdump-a .testdump-b; then exit 1; fi; exit 0"
	@./$(package) -d test.db -R < .testdump-a
	@rm -f .testdump-a .testdump-b .testdump-c
	@echo '	[OK]'
	@bash -c "echo -n ' 08: Automatic database pruning (2)'"
	@./$(package) -d test.db -D | sed 's/SINCEPRUNE.*/SINCEPRUNE 1/' \
	 | grep SINCEPRUNE > .testdump-b
	@./$(package) -d test.db -D | sed 's/SINCEPRUNE.*/SINCEPRUNE 500/' > .testdump-a
	@./$(package) -d test.db -R < .testdump-a
	@formail +1 -1 -s cat < .test-non-spam \
	 | ./$(package) -d test.db -m >/dev/null
	@formail +1 -1 -s cat < .test-non-spam \
	 | ./$(package) -d test.db -m >/dev/null
	@./$(package) -d test.db -D | grep SINCEPRUNE > .testdump-a
	@cmp -s .testdump-a .testdump-b
	@rm -f .testdump-a .testdump-b
	@echo '	[OK]'
	@bash -c "echo -n ' 09: GTUBE'"
	@echo > .testdump-a
	@echo 'XJS*C4JDBQADN1.NSBN3*2IDNEN*GTUBE-STANDARD-ANTI-UBE-TEST-EMAIL*C.34X' >> .testdump-a
	@./$(package) -d test.db -s < .testdump-a > .testdump-b
	@grep SPAM .testdump-b >/dev/null
	@rm -f .testdump-a .testdump-b
	@echo '				[OK]'
	@rm -f test.db

leaktest: $(package) .test-spam
	@rm -f test.db
	MALLOC_CHECK_=2 MALLOC_TRACE=trace \
	  ./$(package) -d test.db -T .test-spam .test-non-spam
	mtrace ./$(package) trace
	@rm -f trace test.db
	@echo '*** OK ***'

install: all doc
	$(srcdir)/autoconf/scripts/mkinstalldirs \
	  "$(DESTDIR)/$(bindir)"
	$(srcdir)/autoconf/scripts/mkinstalldirs \
	  "$(DESTDIR)/$(mandir)/man1"
	$(INSTALL) -m 755 $(package) \
	  "$(DESTDIR)/$(bindir)/$(package)"
	$(INSTALL) -m 644 doc/quickref.1 \
	  "$(DESTDIR)/$(mandir)/man1/$(package).1"
	$(DO_GZIP) "$(DESTDIR)/$(mandir)/man1/$(package).1"      || :

uninstall:
	$(UNINSTALL) "$(DESTDIR)/$(bindir)/$(package)"
	$(UNINSTALL) "$(DESTDIR)/$(mandir)/man1/$(package).1"
	$(UNINSTALL) "$(DESTDIR)/$(mandir)/man1/$(package).1.gz"

rpm: dist
	echo macrofiles: `rpm --showrc \
	  | grep ^macrofiles \
	  | cut -d : -f 2- \
	  | sed 's,^[^/]*/,/,'`:`pwd`/rpmmacros > rpmrc
	echo %_topdir `pwd`/rpm > rpmmacros
	rm -rf rpm
	mkdir rpm
	mkdir rpm/SPECS rpm/BUILD rpm/SOURCES rpm/RPMS
	-grep -hsv ^macrofiles /usr/lib/rpm/rpmrc /etc/rpmrc $$HOME/.rpmrc \
	  >> rpmrc
	rpmbuild $(RPMFLAGS) --rcfile=rpmrc -tb $(package)-$(version).tar.gz
	mv rpm/RPMS/*/$(package)-*.rpm .
	rm -rf rpm rpmmacros rpmrc

srpm: dist
	echo macrofiles: `rpm --showrc \
	  | grep ^macrofiles \
	  | cut -d : -f 2- \
	  | sed 's,^[^/]*/,/,'`:`pwd`/rpmmacros > rpmrc
	echo %_topdir `pwd`/rpm > rpmmacros
	rm -rf rpm
	mkdir rpm
	mkdir rpm/SPECS rpm/BUILD rpm/SOURCES rpm/RPMS rpm/SRPMS
	grep -hsv ^macrofiles /usr/lib/rpm/rpmrc /etc/rpmrc $$HOME/.rpmrc \
	  >> rpmrc
	rpmbuild $(RPMFLAGS) --rcfile=rpmrc -ts $(package)-$(version).tar.gz
	mv rpm/SRPMS/*$(package)-*.rpm .
	rm -rf rpm rpmmacros rpmrc

deb: dist
	rm -rf BUILD-DEB
	mkdir BUILD-DEB
	cd BUILD-DEB && tar xzf ../$(package)-$(version).tar.gz
	cd BUILD-DEB && cd $(package)-$(version) && ./debian/rules binary
	mv BUILD-DEB/*.deb .
	rm -rf BUILD-DEB

release: dist rpm srpm
	zcat $(package)-$(version).tar.gz | bzip2 > $(package)-$(version).tar.bz2
